////////////////////////////////////////////////////////////////////
//PHY control module
//
// this module is the sub module within FPGA top that contains
// all the phy related interface circuits

// use `include to define common phy related constants
`include "phy_const.v"

module phy_cntrl (// MII data interface pins
		  TXD0_, TXD1_, TXD2_, TXD3_,
		  TX_EN_, TX_ER_, TX_CLK_, 
		  RXD0_, RXD1_, RXD2_, RXD3_,
		  RX_DV_, RX_ER_, RX_CLK_,
		  COL_, CRS_,
		  // MII Control Interface Pins
		  MDIO, MDINT, MDC, MDDIS, TRSTE_,
		  // Hardware Control Interface
		  CFG_, FDE, BYPSCR, AUTOENA,
		  // Misc
		  ADD_, RESET_OUT, PWRDN,
		  // LED Indicator
		  LED0_, LED1_, LED2_, LED3_,
		  LEDENA, LEDCLK, LEDDAT,
		  // LED Output
		  RJ45_TRC, RJ45_BRC,//2 bits
		  RJ45_TLC, RJ45_BLC, //2 bits
		  // Others
		  RESET_IN, MSG, CHANNEL,
		  DEBUG
		  );
   ////////////////////////////////////////////////////////////////
   // MII data interface pins
   //
   // Trasmit Data
   output [3:0] TXD0_;
   output [3:0] TXD1_;
   output [3:0] TXD2_;
   output [3:0] TXD3_;
   // Transmit Enable [port]
   output [3:0] TX_EN_;
   // Transmit Code Error [port]
   // active High, forces port to transmit Halt (H) code group
   output [3:0] TX_ER_;
   // Transmit Clock [port]; 25Mhz for 100Mbps
   // Synchronize all TX to this; (default: sampled on rising edge)
   input [3:0] 	TX_CLK_;
   // Receive Data: RXD#_[*] #:port  *:wire
   input [3:0] 	RXD0_;/* synthesis syn_useioff = 1 */
   input [3:0] 	RXD1_;/* synthesis syn_useioff = 1 */
   input [3:0] 	RXD2_;/* synthesis syn_useioff = 1 */
   input [3:0] 	RXD3_; /* synthesis syn_useioff = 1 */  
   // Receive Data Valid [port]
   // active high, indicates received code maps to valid data
   input [3:0] 	RX_DV_;/* synthesis syn_useioff = 1 */
   // Receive Error [port]
   // indicate received code group is invalid or PLL not locked
   input [3:0] 	RX_ER_;/* synthesis syn_useioff = 1 */
   // Receive Clock [port]
   input [3:0]	RX_CLK_;
   // Collision Detected [port] active high, asynchronus
   // inactive in full-duplex mode.
   input [3:0] 	COL_;
   // Carrier Sense [port] active high
   // half-duplex: asserted if transmit or receive medium is non-idle
   // full-duplex: assedted if only when transmit medium is non-idle
   input [3:0] 	CRS_;

   ////////////////////////////////////////////////////////////////
   // MII Control Interface Pins

   // Management DATA Input/Output
   inout 	MDIO;
   // Management Data Interrupt: Active low
   // indicate status change, cleared by reading Register 1, then 18
   input 	MDINT;
   // Management Data Clock: Clock for MDIO
   output 	MDC;
   // Management Disable
   // When high: MDIO is readonly
   // When low: bit control reverts to MDIO after reset or powerup cycle
   output 	MDDIS;
   // Tristate [port]: bit controls 0.10 (Isolate bit)
   // when high, isolate a port from MII Data Interface
   output [3:0] TRSTE_;
   
   ////////////////////////////////////////////////////////////////
   // Hardware Control Interface   
   //
   // CFG0: when A/N is enabled Low->High transition causes 
   //       auto-negotiate to resetart.
   //       when A/N is disabled this input selects the operating
   //       and directly affects bit 0.13 100Mbps=1 10Mbps=0
   // CFG1: When A/N is enabled determines operating speed 
   //       advertisement capabilities in combination with CFG2 
   //       and FDE on all ports.
   //       When A/N is disabled, enables 10 Mbps link test and 
   //       directly affects bit 19.8.
   //       High, 10 Mbps link test is disabled and bit 19.8 = 1.
   //       Low, 10 Mbps link test is enabled and bit 19.8 = 0.
   // CFG2: When A/N is enabled, determines operating speed 
   //       advertisement capabilities in combination with CFG1 
   //       on all ports.
   //       When A/N is disabled, selects either TP or FX interface.
   //       Low, TP is enabled and bit 19.2 = 0.
   //       High, FX is enabled and bit 19.2 = 1.
   output [2:0] CFG_;
   // Full-Duplex Enable: active high, affext all ports
   output 	FDE;
   // Bypass Scrambler: In TP mode enables or bypasses Scrambler 
   // operation and directly affects MDIO register bit 19.3
   // When High, Scrambler is bypassed and bit 19.3 = 1.
   // When Low, Scrambler is enabled and bit 19.3 = 0.
   output 	BYPSCR;
   // Auto-Negotiation Enable. active high,
   // enables auto-negotiation on all ports.
   output 	AUTOENA;
   
   ////////////////////////////////////////////////////////////////
   // Misc
   //
   // Address: Set upper three bits of PHY address. ADD<1:0> are set
   // internally to match port number
   output [4:2] ADD_;
   // RESET: active low, OR'ed with the control register Reset bit (0.15)
   output 	RESET_OUT;
   // Power Down: active High, forces LXT974/975 into power down mode.
   // This pin is OR'ed with the Power Down bit (0.11).
   output 	PWRDN;
      
   ////////////////////////////////////////////////////////////////
   // LED Indicator
   //
   // LED#_[*]: #:port *:wire active low,
   // 0: (default) transmitter active.
   // 1: (default) receiver active.
   // 2: (default) link up
   input [2:0]	LED0_;
   input [2:0]	LED1_;
   input [2:0]	LED2_;
   input [2:0]	LED3_;
   // LED Enable: active high, indicates that LEDDAT is active
   input 	LEDENA;
   // 25Mhz clock for LED serial data output
   input 	LEDCLK;
   // LED Data: Serial data output for 24 LEDs
   input 	LEDDAT;
   //LED on ethernet Connector 
   output [1:0] RJ45_TRC; 
   output [1:0] RJ45_BRC; 
   output [1:0] RJ45_TLC;
   output [1:0] RJ45_BLC;
   ////////////////////////////////////////////////////////////////
   // Other IOs dealing with other circuits in the FPGA
   input 	RESET_IN;
   output [63:0] MSG;
   input [7:0] 	 CHANNEL;
   inout [31:0]  DEBUG;
   
   ////////////////////////////////////////////////////////////////
   // setting global control signals
   // this specifies the operating mode of the Phy
   ////////////////////////////////////////////////////////////////
   assign MDC = 1'b0; //need 2.5MHz clock
   assign MDDIS = 1'b1; ////disables MDIO
   assign TRSTE_ = `TRSTE_all_active;
   // refer to table 16 on LX975 datasheet for CFG_ values
   assign CFG_ = `CFG_force_100T_duplex;
   assign FDE = 1'b1;
   assign BYPSCR = 1'b0;
   assign AUTOENA = 1'b0;
   assign ADD_ = `phy_address;
   assign RESET_OUT = ~RESET_IN;
   assign PWRDN = 1'b0;

   ///////////////////////////////////////////////////////
   // Assigns LED from Phy chip to LED on RJ45 connector
   // Green=link up, yellow=activity
   // note LED signals are active low
   assign RJ45_TRC = {LED3_[1] & LED3_[0], LED3_[2]};
   assign RJ45_BRC = {LED2_[1] & LED2_[0], LED2_[2]};   
   assign RJ45_TLC = {LED1_[1] & LED1_[0], LED1_[2]};
   assign RJ45_BLC = {LED0_[1] & LED0_[0], LED0_[2]}; 
  
   /////////////////////////////////////////////////
   //code for registering the RX data at the
   //IO pads for better synchronization
   reg [3:0] rxd3_, rxd2_, rxd1_, rxd0_;
   reg [3:0] rx_dv_, rx_er_;
   wire      buf_clk;
   
   always @ (posedge buf_clk) begin
      rxd0_ <= RXD0_;
      rxd1_ <= RXD1_;
      rxd2_ <= RXD2_;
      rxd3_ <= RXD3_;
      rx_dv_ <= RX_DV_;
      rx_er_ <= RX_ER_;
   end

   ////////////////////////////////////////////////
   reg   rx_dv, rx_er, rx_clk;
   reg [3:0] rx_d;
   //specifies a global clock buffer for the rx_clk
   //this ensures low clock skew
   BUFG bufg_inst(.I(rx_clk), .O(buf_clk));
   
   /////////////////////////////////////////////////
   //mux the 4 incomming RX ports into 1 port
   //using the 2 LSb of CHANNEL as the select lines

   always @ (/*AUTOSENSE*/CHANNEL or RX_CLK_ or rx_dv_ or rx_er_
	     or rxd0_ or rxd1_ or rxd2_ or rxd3_)
     case (CHANNEL[1:0])
       0:begin
	  rx_d <=rxd0_;
	  rx_clk <= RX_CLK_[0];
	  rx_er <= rx_er_[0];
	  rx_dv <= rx_dv_[0];	  
       end
       1:begin
	  rx_d <=rxd1_;
	  rx_clk <= RX_CLK_[1];
	  rx_er <= rx_er_[1];
	  rx_dv <= rx_dv_[1];	  
       end
       2:begin
	  rx_d <=rxd2_;
	  rx_clk <= RX_CLK_[2];
	  rx_er <= rx_er_[2];
	  rx_dv <= rx_dv_[2];	  
       end
       3:begin
	  rx_d <=rxd3_;
	  rx_clk <= RX_CLK_[3];
	  rx_er <= rx_er_[3];
	  rx_dv <= rx_dv_[3];	  
       end
     endcase // case(CHANNEL[1:0])

   ///////////////////////////////////////////////////
   //the receive module is the circuit that actually
   //handles the incomming RX data
   receiver recv(.RX_DV(rx_dv),
		 .RX_ER(rx_er),
		 .RX_CLK(rx_clk),
		 .RX_D(rx_d),
		 .CHANNEL(CHANNEL),
		 .MSG(MSG),
		 .RESET(RESET_IN),
		 .DEBUG(DEBUG));
   

endmodule
